#pragma once

namespace NLeoHelpers
{
	bool LeetRegQueryValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName, REGSAM sam,
						   DWORD *pdwType, void **plpData, DWORD *pcbData);

	// Returns false on failure. Use GetLastError().
	// Uses LeetRegQueryValue() to get you a TCHAR string without having to worry about buffers, types and so on.
	bool LeetRegQueryStringValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName, REGSAM sam, std::basic_string<TCHAR> *pString);

	// strncpy doesn't always null term the destination... sigh.
	// This returns the index of the null terminator.
	int StringCopy(TCHAR *dest, const TCHAR *source, int destSizeChars);

	// _snprintf doesn't always null term the destination... sigh
	// Returns the index of the null terminator.
	int StringFormat(TCHAR *buffer, size_t bufferSizeChars, const TCHAR *format, ...);

	// _vsnprintf doesn't always null term the destination... sigh
	// Returns the index of the null terminator.
	int VStringFormat(TCHAR *buffer, size_t bufferSizeChars, const TCHAR *format, va_list args);

	// delete[] the result.
	// Returns NULL on failure.
	TCHAR *StringAllocAndFormat(const TCHAR *format, ...);

	// delete[] the result.
	// Returns NULL on failure.
	TCHAR *VStringAllocAndFormat(const TCHAR *format, va_list args);

	// You must delete[] the result of this function after use.
	// A path with no slashes or the empty string returns the empty string (you must still delete the result).
	TCHAR *GetParentPath(const TCHAR *path);

	// You must delete[] the result of this function after use.
	TCHAR *JoinPaths(const TCHAR *first, const TCHAR *second);

	// Returns NULL on failure (use GetLastError())
	// delete[] the result when finished with it.
	WCHAR *LeetMBtoWC(const char *mbString);

	// Returns NULL on failure (use GetLastError())
	// delete[] the result when finished with it.
	char *LeetWCtoMB(const WCHAR *wcString);

	const TCHAR *GetLastPathPart(const TCHAR *szIn);
};
